@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 1997-2002 Motonori Nakamura <motonori@media.kyoto-u.ac.jp>
- * Copyright (c) 1997-2002 WIDE Project
- * Copyright (c) 1997-2002 Kyoto University
+ * Copyright (c) 1997-2003 Motonori Nakamura <motonori@media.kyoto-u.ac.jp>
+ * Copyright (c) 1997-2003 WIDE Project
+ * Copyright (c) 1997-2003 Kyoto University
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,7 @@
  */
 
 #ifndef lint
-static char *_id_ = "$Id: smtp.c,v 1.149 2002/02/23 10:19:49 motonori Exp $";
+static char *_id_ = "$Id: smtp.c,v 1.155 2003/08/20 05:12:58 motonori Exp $";
 #endif
 
 # include "common.h"
@@ -76,7 +76,7 @@ sig_term()
 {
 	log(LOG_INFO, "SIGTERM received: semi graceful termination");
 	markstatus(env.rcpt_list, SMTP_TEMPFAIL(51),
-		"Aborted by INT signal", 1);
+		"Aborted by INT signal", 1, 0);
 	longjmp(jmp_env, 1);
 }
 
@@ -129,7 +129,7 @@ deliver()
 			{
 				log(LOG_INFO,
 					"recipients/transaction changed to 1");
-				cnf.rcpts_trans == 1;
+				cnf.rcpts_trans = 1;
 			}
 #if 0
 			else
@@ -237,7 +237,8 @@ static struct	{
 	{smtp_quit_r,	smtp_flush,	 2 MINUTES,	"FAILQUIT"},	/* 14 */
 	{NULL,		NULL,		 0 MINUTES,	"FAIL"},	/* 15 */
 };
-#define STATE(n)	state_tbl[n].name
+
+#define STATE(n)	((n<0)?"UNKNOWN":(n>15)?"UNKNOWN":state_tbl[n].name)
 
 void
 set_timeout(opt)
@@ -350,7 +351,7 @@ smtp()
 			/* DNS lookup failure or something else */
 			if (msg == NULL)
 				msg = "may be DNS problem";
-			markstatus(rcptp, SMTP_TEMPFAIL(52), msg, 0);
+			markstatus(rcptp, SMTP_TEMPFAIL(52), msg, 0, 0);
 			continue;
 		}
 		if (rcptp->domain == NULL)
@@ -527,7 +528,7 @@ smtp()
 			if (bad_state++ > 5)
 			{
 				markstatus(env.rcpt_list, SMTP_TEMPFAIL(51),
-					"No active sockets", 1);
+					"No active sockets", 1, 0);
 				dump_internal();
 				return;
 			}
@@ -662,7 +663,7 @@ new_transaction()
 						rc = SMTP_ERROR(52);
 					else
 						rc = SMTP_TEMPFAIL(52);
-					markstatus(rcptp, rc, msg, 0);
+					markstatus(rcptp, rc, msg, 0, 0);
 					rcptp->stat = RCPT_RETRY;
 				}
 				rcptp = rcptp->dom_chain;
@@ -739,7 +740,7 @@ new_transaction()
 #endif
 				rcode = SMTP_ERROR(50);
 		}
-		markstatus(conn->rcpt, rcode, msgbuf, 1);
+		markstatus(conn->rcpt, rcode, msgbuf, 1, 0);
 
 		close_transaction(conn, 1);
 		return 0;
@@ -868,11 +869,12 @@ int timeout_only, true_term;
 }
 
 static void
-markstatus(rcptp, rcode, response, tracechain)
+markstatus(rcptp, rcode, response, tracechain, retryMX)
 struct recipient *rcptp;
 int rcode;
 char *response;
 int tracechain;
+int retryMX;
 {
 	char hbuf[MAXLINE], rbuf[MAXLINE], pbuf[64];
 	char addrbuf[SYSLOG_BUFSIZE];
@@ -1019,7 +1021,8 @@ int tracechain;
 
 	while (rcptp != NULL)
 	{
-		if (rcptp->result != 0 && !IS4xx(rcptp->result))
+		if (rcptp->result != 0 && !IS4xx(rcptp->result)
+		 || rcptp->stat != RCPT_WORK)
 		{
 			if (tracechain == 0)
 				break;
@@ -1059,7 +1062,14 @@ int tracechain;
 		{
 			struct mx *mxp = rcptp->domain->curmx;
 
-			if (!Terminate && mxp != NULL
+			if (retryMX)
+			{
+				rcptp->stat = RCPT_RETRY;
+				if (cnf.debug & DEBUG_TRANS)
+				log(LOG_DEBUG, "tempfail; retry: %s",
+					rcptp->address);
+			}
+			else if (!Terminate && mxp != NULL
 			 && (mxp->next != NULL
 			  || (mxp->host != NULL && mxp->host->current != NULL
 			   && mxp->host->current->next != NULL)))
@@ -1328,7 +1338,7 @@ int next;	/* true: forward MX for next trial */
 
 	for (rcptp = conn->rcpt; rcptp != NULL; rcptp = rcptp->chain)
 	{
-		if (!Terminate && (rcptp->stat == RCPT_DONE))
+		if (!Terminate && (rcptp->stat != RCPT_WORK))
 			continue;
 		rcptp->stat = RCPT_RETRY;
 		domp = rcptp->domain;
@@ -1439,7 +1449,8 @@ struct domain *domp;
 			}
 			if (domp->rcpt_tail == rcptp)
 				domp->rcpt_tail = rcptpp;
-		} else
+		}
+		else
 			rcptpp = rcptp;
 		rcptp = rcptp->dom_chain;
 		if (rcpttmp->stat == RCPT_DONE)	/* XXX */
@@ -1622,7 +1633,7 @@ struct connection *conn;
 		conn->host->stat = SMTP_TEMPFAIL(52);
 		conn->host->response = "Connection was not established";
 		markstatus(conn->rcpt, conn->host->stat,
-			conn->host->response, 1);
+			conn->host->response, 1, 0);
 		return -1;
 	}
 
@@ -1826,7 +1837,7 @@ struct connection *conn;
 			conn->host->state = STAT_FAIL;
 			close (sd);
 			markstatus(conn->rcpt, SMTP_TEMPFAIL(52),
-				"source port binding failure", 1);
+				"source port binding failure", 1, 0);
 			return -1;
 		}
 	}
@@ -1842,7 +1853,7 @@ struct connection *conn;
 		conn->host->state = STAT_FAIL;
 		close (sd);
 		markstatus(conn->rcpt, SMTP_TEMPFAIL(52),
-			"FIONBIO failed", 1);
+			"FIONBIO failed", 1, 0);
 		return -1;
 	}
 #else
@@ -1853,7 +1864,7 @@ struct connection *conn;
 		conn->host->state = STAT_FAIL;
 		close (sd);
 		markstatus(conn->rcpt, SMTP_TEMPFAIL(52),
-			"F_GETFL failed", 1);
+			"F_GETFL failed", 1, 0);
 		return -1;
 	}
 # ifdef O_NONBLOCK
@@ -1868,7 +1879,7 @@ struct connection *conn;
 		conn->host->state = STAT_FAIL;
 		close (sd);
 		markstatus(conn->rcpt, SMTP_TEMPFAIL(52),
-			"F_SETFL failed", 1);
+			"F_SETFL failed", 1, 0);
 		return -1;
 	}
 #endif
@@ -1884,7 +1895,7 @@ struct connection *conn;
 		conn->host->current->stat = EX_TEMPFAIL;
 		close (sd);
 		markstatus(conn->rcpt, SMTP_TEMPFAIL(52),
-			"connect() failed", 1);
+			"connect() failed", 1, 0);
 		/* try next address of the same MX */
 		conn->host->current = conn->host->current->next;
 #if 0
@@ -1966,7 +1977,7 @@ struct connection *conn;
 		  default:
 			errmsg = smtpstrerror(errno);
 		}
-		markstatus(conn->rcpt, SMTP_TEMPFAIL(52), errmsg, 1);
+		markstatus(conn->rcpt, SMTP_TEMPFAIL(52), errmsg, 1, 0);
 		if (conn->host->current != NULL) /* XXX for safe access */
 			conn->host->current = conn->host->current->next;
 		if (Terminate)
@@ -2225,7 +2236,7 @@ struct connection *conn;
 		conn->host->state = STAT_FAIL;
 		conn->host->stat = SMTP_TEMPFAIL(52);
 		markstatus(conn->rcpt, conn->host->stat,
-			conn->host->response, 1);
+			conn->host->response, 1, 0);
 		return -1;
 	}
 	return 0;
@@ -2268,7 +2279,7 @@ int *rest;
 			conn->host->state = STAT_FAIL;
 			conn->host->stat = SMTP_TEMPFAIL(52);
 			markstatus(conn->rcpt, conn->host->stat,
-				conn->host->response, 1);
+				conn->host->response, 1, 0);
 			return -1;
 		}
 		if (rcode == 0)
@@ -2321,7 +2332,7 @@ int *rest;
 		conn->host->response = newstr(msgbuf);
 		conn->host->state = STAT_FAIL;
 		markstatus(conn->rcpt, conn->host->stat,
-			conn->host->response, 1);
+			conn->host->response, 1, 0);
 		return -1;
 	}
 	return stat;
@@ -2356,7 +2367,7 @@ int flush;
 			conn->host->response
 				= "out of memory for sending buffer";
 			markstatus(conn->rcpt, conn->host->stat,
-				conn->host->response, 1);
+				conn->host->response, 1, 0);
 			return -1;
 		}
 		strcpy(p, buf);
@@ -2378,7 +2389,7 @@ int flush;
 		conn->host->state = STAT_FAIL;
 		conn->host->stat = SMTP_TEMPFAIL(52);
 		markstatus(conn->rcpt, conn->host->stat,
-			conn->host->response, 1);
+			conn->host->response, 1, 0);
 		return -1;
 	}
 	return rcode;
@@ -2462,7 +2473,7 @@ struct connection *conn;
 		conn->host->state = STAT_CLOSED;
 		if (errmsg != NULL)
 		{
-			markstatus(conn->rcpt, SMTP_TEMPFAIL(52), errmsg, 1);
+			markstatus(conn->rcpt, SMTP_TEMPFAIL(52), errmsg, 1, 0);
 			if (conn->host->current != NULL /* XXX for safe access */
 			 && conn->host->current->next == NULL
 			 && conn->host->response == NULL)
@@ -2481,7 +2492,7 @@ struct connection *conn;
 		{
 			sti.ntimeout++;
 			markstatus(conn->rcpt, SMTP_TEMPFAIL(51),
-				"Processing timed out (greeting)", 1);
+				"Processing timed out (greeting)", 1, 0);
 			smtp_quit_s(conn);
 			return -1;
 		}
@@ -2604,9 +2615,11 @@ struct connection *conn;
 				conn->seq, conn->socket, smtpstrerror(errno),
 				conn->host->name, STATE(savestate));
 		}
-		conn->host->current->stat = EX_TEMPFAIL;
+		if (conn->host->current != NULL)
+			conn->host->current->stat = EX_TEMPFAIL;
 		smtp_close(conn->socket);
-		if (conn->host->current->next == NULL)
+		if (conn->host->current == NULL
+		 || conn->host->current->next == NULL)
 			conn->host->state = STAT_FAIL;
 		else
 			conn->host->state = STAT_CLOSED;
@@ -2646,7 +2659,7 @@ struct connection *conn;
 				conn->host->stat = SMTP_ERROR(53);
 				conn->host->response = "Config error: mail loops back to me (MX problem?)";
 				markstatus(conn->rcpt, conn->host->stat,
-					conn->host->response, 1);
+					conn->host->response, 1, 0);
 				smtp_quit_s(conn);
 				return -1;
 			}
@@ -2655,7 +2668,7 @@ struct connection *conn;
 		{
 			sti.ntimeout++;
 			markstatus(conn->rcpt, SMTP_TEMPFAIL(51),
-				"Processing timed out (HELO)", 1);
+				"Processing timed out (HELO)", 1, 0);
 			smtp_quit_s(conn);
 			return -1;
 		}
@@ -2666,7 +2679,7 @@ struct connection *conn;
 			conn->host->stat = SMTP_TEMPFAIL(51);
 			conn->host->response = "Forced temporary failure";
 			markstatus(conn->rcpt, conn->host->stat,
-				 conn->host->response, 1);
+				 conn->host->response, 1, 0);
 			if (conn->reuse)
 				return -1;	/* without QUIT */
 			if (cnf.smtp_reuse && may_be_reused(conn))
@@ -2832,7 +2845,7 @@ struct connection *conn;
 	conn->host->state = STAT_FAIL;
 	conn->host->stat = SMTP_ERROR(54);
 	conn->host->response = "Sender address too long";
-	markstatus(conn->rcpt, conn->host->stat, conn->host->response, 1);
+	markstatus(conn->rcpt, conn->host->stat, conn->host->response, 1, 0);
 	smtp_quit_s(conn);
 	return -1;
 }
@@ -2858,9 +2871,11 @@ struct connection *conn;
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
-		conn->host->current->stat = EX_TEMPFAIL;
+		if (conn->host->current != NULL)
+			conn->host->current->stat = EX_TEMPFAIL;
 		smtp_close(conn->socket);
-		if (conn->host->current->next == NULL)
+		if (conn->host->current == NULL
+		 || conn->host->current->next == NULL)
 			conn->host->state = STAT_FAIL;
 		else
 			conn->host->state = STAT_CLOSED;
@@ -2878,7 +2893,7 @@ struct connection *conn;
 		{
 			sti.ntimeout++;
 			markstatus(conn->rcpt, SMTP_TEMPFAIL(51),
-				"Processing timed out (MAIL)", 1);
+				"Processing timed out (MAIL)", 1, 0);
 			smtp_quit_s(conn);
 			return -1;
 		}
@@ -2903,7 +2918,7 @@ struct connection *conn;
 		else
 			rstr = newstr(buf + 4);
 	}
-	markstatus(conn->rcpt, rcode, rstr, 1);	/* XXX */
+	markstatus(conn->rcpt, rcode, rstr, 1, 0);	/* XXX */
 	conn->host->response = rstr;
 	conn->host->stat = rcode;
 
@@ -2993,7 +3008,7 @@ struct connection *conn;
 	conn->host->state = STAT_FAIL;
 	conn->host->stat = SMTP_ERROR(54);
 	conn->host->response = "Recipient address too long";
-	markstatus(conn->rcpt, conn->host->stat, conn->host->response, 1);
+	markstatus(conn->rcpt, conn->host->stat, conn->host->response, 1, 0);
 	smtp_quit_s(conn);	/* XXX */
 	return -1;
 }
@@ -3036,15 +3051,21 @@ struct connection *conn;
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
-		conn->host->current->stat = EX_TEMPFAIL;
+		if (conn->host->current != NULL)
+			conn->host->current->stat = EX_TEMPFAIL;
 		smtp_close(conn->socket);
-		if (conn->host->current->next == NULL)
+		if (conn->host->current == NULL
+		 || conn->host->current->next == NULL)
 			conn->host->state = STAT_FAIL;
 		else
 			conn->host->state = STAT_CLOSED;
 		return -1;
 	}
-
+#if 1
+	/* 552: RFC821 too may recipients should be tempfail */
+	if (rcode == SMTP_ERROR(52) && strstr(buf, "oo many recipient") != NULL)
+		rcode = SMTP_TEMPFAIL(52);
+#endif
 	if (IS2xx(rcode))
 		conn->goodrcpt = 1;
 	else if ((conn->host->proto & PROTO_PIPELINING) == 0
@@ -3052,6 +3073,30 @@ struct connection *conn;
 	{
 		char tmpbuf[MAXLINE], *rstr;
 
+		if (rcode == SMTP_TEMPFAIL(52))
+		{
+			/* too many recipients */
+
+			if (!conn->reuse)
+				conn->reuse = 1;
+			/* conn->crcptr->stat = RCPT_RETRY; */
+
+			if ((conn->host->proto & PROTO_PIPELINING) == 0)
+			{
+				/* preserve rest of recipients for next
+				   transaction */
+				struct recipient *tmprcptp;
+
+				tmprcptp = conn->crcptr->chain;
+				while (tmprcptp != NULL)
+				{
+					tmprcptp->stat = RCPT_RETRY;
+					tmprcptp = tmprcptp->chain;
+				}
+				conn->crcptr->chain = NULL;
+			}
+		}
+
 		if (!(IS4xx(rcode) || IS5xx(rcode)))
 		{
 			rcode = SMTP_ERROR(54);
@@ -3068,7 +3113,10 @@ struct connection *conn;
 				rstr = newstr(skipaddr(buf + 4,
 					conn->crcptr->address));
 		}
-		markstatus(conn->crcptr, rcode, rstr, 0);
+		if (rcode != SMTP_TEMPFAIL(52) || conn->rcpt == conn->crcptr)
+			markstatus(conn->crcptr, rcode, rstr, 0, 0);
+		else
+			markstatus(conn->crcptr, rcode, rstr, 0, 1);
 	}
 	conn->crcptr = conn->crcptr->chain;
 
@@ -3076,7 +3124,7 @@ struct connection *conn;
 	{
 		sti.ntimeout++;
 		markstatus(conn->rcpt, SMTP_TEMPFAIL(51),
-			"Processing timed out (RCPT)", 1);
+			"Processing timed out (RCPT)", 1, 0);
 		smtp_quit_s(conn);
 		return -1;
 	}
@@ -3094,8 +3142,11 @@ struct connection *conn;
 		else
 		{
 			/* failure on all recipients */
-			/* conn->host->state = STAT_FAIL; */
-			smtp_quit_s(conn);	/* XXX */
+			conn->host->state = STAT_SENT;
+			sti.noktrans++; /* XXX */
+			if (cnf.smtp_reuse && may_be_reused(conn))
+				return -1;	/* without QUIT */
+			smtp_quit_s(conn);
 			return -1;
 		}
 	}
@@ -3153,9 +3204,11 @@ struct connection *conn;
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
-		conn->host->current->stat = EX_TEMPFAIL;
+		if (conn->host->current != NULL)
+			conn->host->current->stat = EX_TEMPFAIL;
 		smtp_close(conn->socket);
-		if (conn->host->current->next == NULL)
+		if (conn->host->current == NULL
+		 || conn->host->current->next == NULL)
 			conn->host->state = STAT_FAIL;
 		else
 			conn->host->state = STAT_CLOSED;
@@ -3164,12 +3217,31 @@ struct connection *conn;
 
 	if (IS3xx(rcode))
 	{
+		if (conn->goodrcpt == 0)
+		{
+			/* no valid recipients, send empty body */
+			char tb[MAXLINE];
+
+			strcpy(tb, ".");
+			if (smtp_send_command(conn, tb, sizeof(tb), 1) < 0)
+			{
+				/* XXX */
+				smtp_close(conn->socket);
+				if (conn->host->current->next == NULL)
+					conn->host->state = STAT_FAIL;
+				else
+					conn->host->state = STAT_CLOSED;
+				return -1;
+			}
+			conn->host->state = STAT_DTERMSENT;
+			return 0;
+		}
 		if (Terminate)
 		{
 			sti.ntimeout++;
 			conn->host->state = STAT_FAIL;
 			markstatus(conn->rcpt, SMTP_TEMPFAIL(51),
-				"Processing timed out (DATA)", 1);
+				"Processing timed out (DATA)", 1, 0);
 			/* XXX how can i terminate here ? */
 			smtp_close(conn->socket);
 			if (conn->host->current->next == NULL)
@@ -3182,6 +3254,19 @@ struct connection *conn;
 		return smtp_data_body(conn);
 	}
 
+	if (rcode == SMTP_ERROR(03) || rcode == SMTP_ERROR(54))
+	{
+		/* there were no valid recipients but we may have more
+		   recipients to attempt */
+
+		conn->host->state = STAT_SENT;
+		sti.noktrans++; /* XXX */
+		if (cnf.smtp_reuse && may_be_reused(conn))
+			return -1;	/* without QUIT */
+		smtp_quit_s(conn);
+		return -1;		/* to close transaction */
+	}
+
 	if (!(IS4xx(rcode) || IS5xx(rcode)))
 	{
 		rcode = SMTP_ERROR(54);
@@ -3200,7 +3285,7 @@ struct connection *conn;
 
 	if ((conn->host->proto & PROTO_PIPELINING) == 0 || conn->goodmail == 1)
 	{
-		markstatus(conn->rcpt, rcode, rstr, 1);	/* XXX */
+		markstatus(conn->rcpt, rcode, rstr, 1, 0);	/* XXX */
 		conn->host->stat = rcode;
 		conn->host->response = rstr;
 	}
@@ -3258,7 +3343,7 @@ struct connection *conn;
 				conn->host->stat = SMTP_TEMPFAIL(51);
 				conn->host->response = "lost connection";
 				markstatus(conn->rcpt, conn->host->stat,
-					conn->host->response, 1);
+					conn->host->response, 1, 0);
 				/* smtp_quit_s(conn); */
 				return -1;
 			}
@@ -3365,9 +3450,11 @@ struct connection *conn;
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
-		conn->host->current->stat = EX_TEMPFAIL;
+		if (conn->host->current != NULL)
+			conn->host->current->stat = EX_TEMPFAIL;
 		smtp_close(conn->socket);
-		if (conn->host->current->next == NULL)
+		if (conn->host->current == NULL
+		 || conn->host->current->next == NULL)
 			conn->host->state = STAT_FAIL;
 		else
 			conn->host->state = STAT_CLOSED;
@@ -3390,7 +3477,10 @@ struct connection *conn;
 			rstr = newstr(buf + 4);
 	}
 
-	markstatus(conn->rcpt, rcode, rstr, 1);
+	if (rcode == 221)
+		rcode = 250;	/* for MTAs which reply bogus code */
+
+	markstatus(conn->rcpt, rcode, rstr, 1, 0);
 
 	if (IS2xx(rcode))
 	{
@@ -3401,10 +3491,10 @@ struct connection *conn;
 		if (cnf.smtp_reuse && may_be_reused(conn))
 			return -1;	/* without QUIT */
 		smtp_quit_s(conn);
-		return -1;	/* to close transaction */
+		return -1;		/* to close transaction */
 	}
 
-	if (rcode == 550 || rcode == 552) {
+	if (rcode == SMTP_ERROR(50) || rcode == SMTP_ERROR(52)) {
 		/* for bogus mail gateway to mobile phone.... */
 		conn->host->state = STAT_CLOSED;
 	} else {
@@ -3457,9 +3547,11 @@ struct connection *conn;
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
-		conn->host->current->stat = EX_TEMPFAIL;
+		if (conn->host->current != NULL)
+			conn->host->current->stat = EX_TEMPFAIL;
 		smtp_close(conn->socket);
-		if (conn->host->current->next == NULL)
+		if (conn->host->current == NULL
+		 || conn->host->current->next == NULL)
 			conn->host->state = STAT_FAIL;
 		else
 			conn->host->state = STAT_CLOSED;
@@ -3472,7 +3564,7 @@ struct connection *conn;
 		{
 			sti.ntimeout++;
 			markstatus(conn->rcpt, SMTP_TEMPFAIL(51),
-				"Processing timed out (RSET)", 1);
+				"Processing timed out (RSET)", 1, 0);
 			smtp_quit_s(conn);
 			return -1;
 		}
@@ -3532,8 +3624,10 @@ struct connection *conn;
 				conn->seq, conn->socket, conn->host->name,
 				smtpstrerror(errno), STATE(savestate));
 		}
-		conn->host->current->stat = EX_TEMPFAIL;
-		if (conn->host->current->next == NULL)
+		if (conn->host->current != NULL)
+			conn->host->current->stat = EX_TEMPFAIL;
+		if (conn->host->current == NULL
+		 || conn->host->current->next == NULL)
 			conn->host->state = STAT_FAIL;
 		else
 			conn->host->state = STAT_CLOSED;
@@ -3586,7 +3680,7 @@ struct connection *conn;
 		 || conn->host->current == NULL /* XXX for safe access */
 		 || conn->host->current->next == NULL)
 			markstatus(conn->rcpt, SMTP_TEMPFAIL(52),
-				newstr(buf), 1);
+				newstr(buf), 1, 0);
 		else
 			log(LOG_INFO, "Trying next address of %s",
 			    conn->host->name);
@@ -3623,7 +3717,7 @@ struct connection *conn;
 			conn->host->stat = EX_TEMPFAIL;
 			if (conn->rcpt != NULL)
 				markstatus(conn->rcpt, SMTP_TEMPFAIL(51),
-					newstr(buf), 1);
+					newstr(buf), 1, 0);
 		}
 		smtp_close(conn->socket);
 		return -1;
